==> phoenix-0.1.7.tar.gz: phoenix-0.1.7/scripts/output.tcl <==
################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

## output.tcl: output for Phoenix user interface
## ==============
## Copyright (C) 1992-1993
## Global Network Academy
## Macvicar Institute for Educational Software Development
##
## See the file COPYRIGHT for conditions

# All output should go through these procedures

proc tkW3OutputSetMessage {message {duration ""}} {
    tkW3OutputEntryPrint .message $message
    if {$duration != ""} {
	after $duration {tkW3OutputSetMessage {}}
    }
}

proc tkW3OutputEntryPrint {w message} {
	set setting [lindex [$w config -state] 4]
	$w configure -state normal

	set message [string trim $message]

	if {[winfo class $w] == "Text"} {
		$w delete 1.0 end
		$w insert 1.0 $message
		$w tag add H1 1.0 end
	} else {
		$w delete 0 end
		$w insert 0 $message
	}
	
	$w configure -state $setting

}

# Some abstraction barriers
# Note that tcl returns the value of the last statement executed

proc tkW3OutputGetAddress {} {
   .titles.address_entry get
}

proc tkW3OutputGetTitle {} {
   .titles.title_entry get 1.0 end
}

proc tkW3OutputSetAddress {address title} {
    tkW3OutputEntryPrint .titles.address_entry $address
    tkW3OutputEntryPrint .titles.title_entry $title

    # Hack to make sure title_entry resizes itself!!
    .titles.title_entry configure -height \
	[expr 2*[lindex [.titles.title_entry configure -height] 4]+1]
    update idletasks
    .titles.title_entry configure -height \
	[expr 2*[lindex [.titles.title_entry configure -height] 4]+2]

    PhxApp_rename_window $title
}

proc tkW3OutputSetBodyToImage {filename} {

	global phxW3HtBuildingDocument

	if {$phxW3HtBuildingDocument} {
		return
	}
	global tkW3ConfigViewer
	exec $tkW3ConfigViewer(image) $filename
}

proc tkW3OutputSetBodyToFile {filename} {

	tkW3HtBeginDoc
	tkW3OutputClearBody
	[phxW3WidgetCurrentText] config -state normal
	set file [open $filename "r"]
	while {[gets $file line] != -1} {
		[phxW3WidgetCurrentText] insert insert "$line\n"
	}
	close $file
	[phxW3CurrentHtext] tag add PRE 1.0 end
	global phxW3ModeTextState
	#[phxW3WidgetCurrentText] config -state $phxW3ModeTextState

	tkW3HtEndDoc
}

proc tkW3OutputGetScrollPosition {} {
    lindex [.vs get] 2
}

# tkW3OutputSetScrollPosition: The scrollbar is update only when
# the yview value changes.  To ensure update, we first set it to
# an incorrect value.

proc tkW3OutputSetScrollPosition {value} {
	[phxW3CurrentHtext] yview [incr value]
	update idletasks
	[phxW3CurrentHtext] yview [incr value -1]
	update idletasks
}

## ************************
## Cursor procedures
## ************************

# tkW3OutputCursorWait
# tkW3OutputCursorNormal
# 
# Be "busy" and "not busy," respectively.

# The number of times tkW3OutputCursorWait has been called.
global phxW3OutputCursorWaits
set phxW3OutputCursorWaits 0

proc tkW3OutputCursorWait {} {

    global phxW3OutputCursorWaits
    incr phxW3OutputCursorWaits
    if {$phxW3OutputCursorWaits != 1} { return }

    global OutputRestoreCmd

    .status configure -text "Please Wait..." -foreground blue -relief groove
    append OutputRestoreCmd ".status configure -text {} -relief flat\n"


    # Grab
    set currentGrab [grab current .]
    if {$currentGrab != ""} {
	switch [grab status $currentGrab] {
	    {local} {
		catch {grab release $currentGrab}
		append OutputRestoreCmd "catch \{grab $currentGrab\}\n"
	    }
	    {global} {
		catch {grab release $currentGrab}
		append OutputRestoreCmd "catch \{grab -global $currentGrab\}\n"
	    }
	}
    } else {
	append OutputRestoreCmd "catch {grab release .status}\n"
    }

    catch {grab .status}

    # Focus
    append OutputRestoreCmd [list \
	if {[focus] == ".status"} "focus [focus]"] "\n"
    focus .status

    # Cursor
    append OutputRestoreCmd "update\n"
    foreach w {. .f} {
	append OutputRestoreCmd \
	    "$w configure -cursor [lindex [$w configure -cursor] 4]\n"
	$w configure -cursor watch
    }

    # Do it now
    update idletasks
    after 500 phxW3OutputStatusFlashFG
}

# phxW3OutputStatusFlashFG
# Alternate "Please Wait" message in blue and red

proc phxW3OutputStatusFlashFG {} {
    if {[lindex [.status configure -text] 4] == "Please Wait..."} {

	switch [lindex [.status config -fg] 4] {
	    {blue} { .status config -fg red }
	    {default} { .status config -fg blue }
	}
#	update idletasks
	after 500 phxW3OutputStatusFlashFG
    }
}

proc phxW3OutputStatusFlashBG {key} {
    if {$key != ""} {
	.status configure -bg red
	update idletasks
	after 100 {
	    .status configure -bg white
#	    update idletasks
	}
    }
}

proc tkW3OutputCursorNormal {} {
    global phxW3OutputCursorWaits
    incr phxW3OutputCursorWaits -1
    if {$phxW3OutputCursorWaits != 0} { return }

    # Make sure any pending mouse or keyboard events are ignored.
    update

    # Restore Grab, Focus, and Cursor
    global OutputRestoreCmd
    eval $OutputRestoreCmd
    set OutputRestoreCmd ""
}

proc tkW3OutputMakeEntries {w list} {
    if [winfo exists $w] {
	foreach child [winfo children $w] {
	    destroy $child
	}
    } {
	frame $w -borderwidth 0
    }

    foreach item $list {
	    label $w.[lindex $item 0]_label -text [lindex $item 1]
	    entry $w.[lindex $item 0] -state normal
#	pack configure \
#	    [label $w.[lindex $item 0]_label -text [lindex $item 1]] -side left
#	pack configure [entry $w.[lindex $item 0] -state normal] \
#		-side left -expand yes -fill x
    }
    return $w
}

proc tkW3OutputMakeFrame {w} {
    if [winfo exists $w] {
	foreach child [winfo children $w] {
	    destroy $child
	}
    } {
	frame $w -borderwidth 0
    }
}


proc tkW3OutputMakeButtons {w list} {
    global fsBox

    tkW3OutputMakeFrame $w

    PhxApp_build_buttonbar $w
    # PhxApp_build_buttonbar creates $w.quit but we're going to redo it.
    # Also, we're going to override -borderwidth and -relief.
    destroy $w.quit
    $w configure -borderwidth 1 -relief sunken

    foreach item $list {

	set name [lindex $item 0]
	set text [lindex $item 1]

	if {[string index $text 0] != "@"} {
	    button $w.$name -borderwidth 1 -text $text \
		-command [lindex $item 2]
	} else {
	    button $w.$name -borderwidth 1 -bitmap $text \
		-command [lindex $item 2]
	}
	if {[lindex $item 3] != {}} {
	    eval $w.$name configure [lindex $item 3]
	}

	# Use eval so that the [lindex ...] becomes several arguments.
	eval pack $w.$name -in $w -side top -fill x [lindex $item 4]
    }

    PhxApp_standardize_buttonbar $w

    return $w
}

proc tkW3OutputDoNothing {} {
}

proc tkW3OutputMakeToggles {w list} {
    tkW3OutputMakeFrame $w
    #foreach item $list {
	#set name [lindex $item 0]
#	pack configure \
#	    [checkbutton $w.$name -text [lindex $item 1] -variable $name] \
#	    -side right
	#bind $w.$name <Any-Enter> "tkW3OutputDoNothing"
	#bind $w.$name <Any-Leave> "tkW3OutputDoNothing"
	#bind $w.$name <1> "tkW3OutputDoNothing"
	#bind $w.$name <ButtonRelease-1> "tkW3OutputDoNothing"
    #}
    #pack append $w 
	entry .message -state disabled 
      #{left  expand fillx}

#	[label .message_label -text "Message:"] {left}

    return $w
}

proc tkW3OutputMakeTitleBox {w} {
    global phxsrcdir
    tkW3OutputMakeFrame $w

    entry $w.address_entry -state normal
    pack configure \
	[button .anchor -relief flat -borderwidth 2 \
	    -bitmap @$phxsrcdir/anchor.icon \
	    -command {phxW3AnchorBroadcastHref ""}] \
	-in $w -side right
    pack configure \
	[text $w.title_entry -state normal -height 1 -relief flat \
	    -yscrollcommand "phxW3WidgetAdjustHeight $w.title_entry"] \
	-side left -fill both -expand yes -anchor center
    phxW3SelectionWidgetInitialize $w.title_entry	

    # We'll grab and focus to .status when we wish to lock out the user.
    bind .status <Any-KeyPress> { phxW3OutputStatusFlashBG %A}
    bind .status <Any-Button> { phxW3OutputStatusFlashBG Button}

    return $w
}

proc tkW3OutputError {message {fatal 0}} {
   if {$fatal != 0} {
       DLG:error . .error $message "Close window"
       tkW3NavigateCloseThisWindow
   } {
       tkW3OutputCursorNormal
       DLG:error . .error $message "OK"
       tkW3OutputCursorWait
   }
}


proc tkW3OutputMakeScrollbars {} {
    scrollbar .vs \
	-command {phxW3WidgetYview .vs [phxW3CurrentHtext]} \
	-orient vertical
    PhxApp_standardize_windowwidg .vs
    .vs configure -borderwidth 1 -relief sunken

    return .vs
}

proc phxW3OutputMakeHtext {htext} {

	catch {frame .f}
	if {![winfo exists $htext]} {
		blt_htext $htext -yscrollcommand {.vs set}
	}

	tkW3EditBind $htext
	bind $htext <Button-1> \
	    {phxW3SelectionSetFocus [phxW3WidgetLeft end] end}
	pack configure $htext -side top -fill both -expand yes

	tkW3OutputClearBody $htext
	return [winfo parent $htext]
}



proc tkW3OutputClearBody {{htext ""}} {
	global phxW3CurrentWidget

	if {$htext == ""} { set htext [phxW3CurrentHtext] }

	$htext config -text ""
	catch {frame $htext.frame -borderwidth 0 -relief flat}
	set phxW3CurrentWidget [phxW3CurrentFrame $htext.frame]
}

proc tkW3OutputShowBody {} {
	phxW3WidgetInitialize
}

proc tkW3OutputSaveFileAs {file_name} {

	tkW3OutputClearBody
	set oldfilename [lindex [split [tkW3OutputGetAddress] :] 0]
	set filename [phxW3Filebox w]
	if {$filename != ""} {
		exec mv $oldfilename $filename
	}
}

proc phxW3OutputRedraw {} {
    foreach x [pack slaves .] {
	append redraw "pack $x " [pack newinfo $x] "\n"
    }
    foreach x [pack slaves .] {
	pack forget $x
    }
    eval $redraw
}

==> phoenix-0.1.8.tar.gz: phoenix-0.1.8/scripts/output.tcl <==
################################################################
# Modifications beyond tkWWW 0.9 are Copyright 1993, 1994 by
# The University of Chicago Biological Sciences Division, Academic Computing
# Philip Stylianos
# Lee Newberg
################################################################

## output.tcl: output for Phoenix user interface
## ==============
## Copyright (C) 1992-1993
## Global Network Academy
## Macvicar Institute for Educational Software Development
##
## See the file COPYRIGHT for conditions

# All output should go through these procedures

proc tkW3OutputSetMessage {message {duration ""}} {
    tkW3OutputEntryPrint .message $message
    if {$duration != ""} {
	after $duration {tkW3OutputSetMessage {}}
    }
}

proc tkW3OutputEntryPrint {w message} {
	set setting [lindex [$w config -state] 4]
	$w configure -state normal

	set message [string trim $message]

	if {[winfo class $w] == "Text"} {
		$w delete 1.0 end
		$w insert 1.0 $message
		$w tag add H1 1.0 end
	} else {
		$w delete 0 end
		$w insert 0 $message
	}
	
	$w configure -state $setting

}

# Some abstraction barriers
# Note that tcl returns the value of the last statement executed

proc tkW3OutputGetAddress {} {
   .titles.address_entry get
}

proc tkW3OutputGetTitle {} {
   .titles.title_entry get 1.0 end
}

proc tkW3OutputSetAddress {address title} {
    tkW3OutputEntryPrint .titles.address_entry $address
    tkW3OutputEntryPrint .titles.title_entry $title

    # Hack to make sure title_entry resizes itself!!
    .titles.title_entry configure -height \
	[expr 2*[lindex [.titles.title_entry configure -height] 4]+1]
    update idletasks
    .titles.title_entry configure -height \
	[expr 2*[lindex [.titles.title_entry configure -height] 4]+2]

    PhxApp_rename_window $title
}

proc tkW3OutputSetBodyToImage {filename} {

	global phxW3HtBuildingDocument

	if {$phxW3HtBuildingDocument} {
		return
	}
	global tkW3ConfigViewer
	exec $tkW3ConfigViewer(image) $filename
}

proc tkW3OutputSetBodyToFile {filename} {

	tkW3HtBeginDoc
	tkW3OutputClearBody
	[phxW3WidgetCurrentText] config -state normal
	set file [open $filename "r"]
	while {[gets $file line] != -1} {
		[phxW3WidgetCurrentText] insert insert "$line\n"
	}
	close $file
	[phxW3CurrentHtext] tag add PRE 1.0 end
	global phxW3ModeTextState
	#[phxW3WidgetCurrentText] config -state $phxW3ModeTextState

	tkW3HtEndDoc
}

proc tkW3OutputGetScrollPosition {} {
    lindex [.vs get] 2
}

# tkW3OutputSetScrollPosition: The scrollbar is update only when
# the yview value changes.  To ensure update, we first set it to
# an incorrect value.

proc tkW3OutputSetScrollPosition {value} {
	[phxW3CurrentHtext] yview [incr value]
	update idletasks
	[phxW3CurrentHtext] yview [incr value -1]
	update idletasks
}

## ************************
## Cursor procedures
## ************************

# tkW3OutputCursorWait
# tkW3OutputCursorNormal
# 
# Be "busy" and "not busy," respectively.

proc tkW3OutputCursorWait {} {

    global phxW3OutputCursorWaits
    if {![info exists phxW3OutputCursorWaits]} { set phxW3OutputCursorWaits 0 }
    incr phxW3OutputCursorWaits
    if {$phxW3OutputCursorWaits != 1} { return }

    global OutputRestoreCmd

    .status configure -text "Please Wait..." -foreground blue -relief groove
    append OutputRestoreCmd ".status configure -text {} -relief flat\n"


    # Grab
    set currentGrab [grab current .]
    if {$currentGrab != ""} {
	switch [grab status $currentGrab] {
	    {local} {
		catch {grab release $currentGrab}
		append OutputRestoreCmd "catch \{grab $currentGrab\}\n"
	    }
	    {global} {
		catch {grab release $currentGrab}
		append OutputRestoreCmd "catch \{grab -global $currentGrab\}\n"
	    }
	}
    } else {
	append OutputRestoreCmd "catch {grab release .status}\n"
    }

    catch {grab .status}

    # Focus
    append OutputRestoreCmd [list \
	if {[focus] == ".status"} "focus [focus]"] "\n"
    focus .status

    # Cursor
    append OutputRestoreCmd "update\n"
    foreach w {. .f} {
	append OutputRestoreCmd \
	    "$w configure -cursor [lindex [$w configure -cursor] 4]\n"
	$w configure -cursor watch
    }

    # Do it now
    update idletasks
    after 500 phxW3OutputStatusFlashFG
}

# phxW3OutputStatusFlashFG
# Alternate "Please Wait" message in blue and red

proc phxW3OutputStatusFlashFG {} {
    if {[lindex [.status configure -text] 4] == "Please Wait..."} {

	switch [lindex [.status config -fg] 4] {
	    {blue} { .status config -fg red }
	    {default} { .status config -fg blue }
	}
#	update idletasks
	after 500 phxW3OutputStatusFlashFG
    }
}

proc phxW3OutputStatusFlashBG {key} {
    if {$key != ""} {
	.status configure -bg red
	update idletasks
	after 100 {
	    .status configure -bg white
#	    update idletasks
	}
    }
}

proc tkW3OutputCursorNormal {} {
    global phxW3OutputCursorWaits
    incr phxW3OutputCursorWaits -1
    if {$phxW3OutputCursorWaits != 0} { return }

    # Make sure any pending mouse or keyboard events are ignored.
    update

    # Restore Grab, Focus, and Cursor
    global OutputRestoreCmd
    eval $OutputRestoreCmd
    set OutputRestoreCmd ""
}

proc tkW3OutputMakeEntries {w list} {
    if [winfo exists $w] {
	foreach child [winfo children $w] {
	    destroy $child
	}
    } {
	frame $w -borderwidth 0
    }

    foreach item $list {
	    label $w.[lindex $item 0]_label -text [lindex $item 1]
	    entry $w.[lindex $item 0] -state normal
#	pack configure \
#	    [label $w.[lindex $item 0]_label -text [lindex $item 1]] -side left
#	pack configure [entry $w.[lindex $item 0] -state normal] \
#		-side left -expand yes -fill x
    }
    return $w
}

proc tkW3OutputMakeFrame {w} {
    if [winfo exists $w] {
	foreach child [winfo children $w] {
	    destroy $child
	}
    } {
	frame $w -borderwidth 0
    }
}


proc tkW3OutputMakeButtons {w list} {
    global fsBox

    tkW3OutputMakeFrame $w

    PhxApp_build_buttonbar $w
    # PhxApp_build_buttonbar creates $w.quit but we're going to redo it.
    # Also, we're going to override -borderwidth and -relief.
    destroy $w.quit
    $w configure -borderwidth 1 -relief sunken

    foreach item $list {

	set name [lindex $item 0]
	set text [lindex $item 1]

	if {[string index $text 0] != "@"} {
	    button $w.$name -borderwidth 1 -text $text \
		-command [lindex $item 2]
	} else {
	    button $w.$name -borderwidth 1 -bitmap $text \
		-command [lindex $item 2]
	}
	if {[lindex $item 3] != {}} {
	    eval $w.$name configure [lindex $item 3]
	}

	# Use eval so that the [lindex ...] becomes several arguments.
	eval pack $w.$name -in $w -side top -fill x [lindex $item 4]
    }

    PhxApp_standardize_buttonbar $w

    return $w
}

proc tkW3OutputDoNothing {} {
}

proc tkW3OutputMakeToggles {w list} {
    tkW3OutputMakeFrame $w
    #foreach item $list {
	#set name [lindex $item 0]
#	pack configure \
#	    [checkbutton $w.$name -text [lindex $item 1] -variable $name] \
#	    -side right
	#bind $w.$name <Any-Enter> "tkW3OutputDoNothing"
	#bind $w.$name <Any-Leave> "tkW3OutputDoNothing"
	#bind $w.$name <1> "tkW3OutputDoNothing"
	#bind $w.$name <ButtonRelease-1> "tkW3OutputDoNothing"
    #}
    #pack append $w 
	entry .message -state disabled 
      #{left  expand fillx}

#	[label .message_label -text "Message:"] {left}

    return $w
}

proc tkW3OutputMakeTitleBox {w} {
    global phxsrcdir
    tkW3OutputMakeFrame $w

    entry $w.address_entry -state normal
    pack configure \
	[button .anchor -relief flat -borderwidth 2 \
	    -bitmap @$phxsrcdir/anchor.icon \
	    -command {phxW3AnchorBroadcastHref ""}] \
	-in $w -side right
    pack configure \
	[text $w.title_entry -state normal -height 1 -relief flat \
	    -yscrollcommand "phxW3WidgetAdjustHeight $w.title_entry"] \
	-side left -fill both -expand yes -anchor center
    phxW3SelectionWidgetInitialize $w.title_entry	

    # We'll grab and focus to .status when we wish to lock out the user.
    bind .status <Any-KeyPress> { phxW3OutputStatusFlashBG %A}
    bind .status <Any-Button> { phxW3OutputStatusFlashBG Button}

    return $w
}

proc tkW3OutputError {message {fatal 0}} {
   if {$fatal != 0} {
       DLG:error . .error $message "Close window"
       tkW3NavigateCloseThisWindow
   } {
       tkW3OutputCursorNormal
       DLG:error . .error $message "OK"
       tkW3OutputCursorWait
   }
}


proc tkW3OutputMakeScrollbars {} {
    scrollbar .vs \
	-command {phxW3WidgetYview .vs [phxW3CurrentHtext]} \
	-orient vertical
    PhxApp_standardize_windowwidg .vs
    .vs configure -borderwidth 1 -relief sunken

    return .vs
}

proc phxW3OutputMakeHtext {htext} {
    catch {frame .f}
    if {![winfo exists $htext]} {
	blt_htext $htext -yscrollcommand {phxW3WidgetScrollbarSet .vs .space2}
    }

    tkW3EditBind $htext
    bind $htext <Button-1> \
	{phxW3SelectionSetFocus [phxW3WidgetLeft end] end}
    pack configure $htext -side top -fill both -expand yes

    tkW3OutputClearBody $htext
    return [winfo parent $htext]
}



proc tkW3OutputClearBody {{htext ""}} {
	global phxW3CurrentWidget

	if {$htext == ""} { set htext [phxW3CurrentHtext] }

	$htext config -text ""
	catch {frame $htext.frame -borderwidth 0 -relief flat}
	set phxW3CurrentWidget [phxW3CurrentFrame $htext.frame]
}

proc tkW3OutputShowBody {} {
	phxW3WidgetInitialize
}

proc tkW3OutputSaveFileAs {file_name} {

	tkW3OutputClearBody
	set oldfilename [lindex [split [tkW3OutputGetAddress] :] 0]
	set filename [phxW3Filebox w]
	if {$filename != ""} {
		exec mv $oldfilename $filename
	}
}

# Refresh the screen by forgetting each top level widget and
# then re-packing it.

proc phxW3OutputRedraw {} {
    foreach x [pack slaves .] {
	append redraw "pack $x " [pack newinfo $x] "\n"
    }
    foreach x [pack slaves .] {
	pack forget $x
    }
    eval $redraw
}

# phxW3OutputPushAndBlankUserOptions:
# Record the status of every button and menubutton.  Then
# gray them out and render them unaccessible.
# (Should separate into two routines if there is any need!)

proc phxW3OutputPushAndBlankUserOptions {} {
    global phxW3OutputUserOptions

    foreach w [PhxUtil_get_descendants $phxW3OutputUserOptions(parents)] {
	switch [winfo class $w] {
	Menu {
	    if {[set last [$w index last]] != "none"} {
		for {set entry 0} {$entry <= $last} {incr entry} {
		    if {![catch {set state [lindex [$w entry $entry -state] 4]}]} {
			append cmds "[list $w entry $entry -state $state]\n"
			$w entry $entry -state disabled
		    }
		}
	    }
	}
	Menubutton -
	Button {
	    append cmds "[list $w conf -state [lindex [$w conf -state] 4]]\n"
	    $w config -state disabled
	}}
    }

    lappend phxW3OutputUserOptions(stack) $cmds
}

# phxW3OutputPopUserOptions: Restore recorded button's and
# menubutton's accessibility

proc phxW3OutputPopUserOptions {} {
    global phxW3OutputUserOptions
    set last [llength $phxW3OutputUserOptions(stack)]
    if {[incr last -1] >= 0} {
	set cmds [lindex $phxW3OutputUserOptions(stack) $last]
	set phxW3OutputUserOptions(stack) \
	    [lreplace $phxW3OutputUserOptions(stack) $last $last]
	eval $cmds
    }
}

# phxW3OutputInitialize: Initialize the stack of UserOptions
# settings to be empty.  Also record the names of the widgets
# containing the menus and the buttons.

proc phxW3OutputInitialize {args} {
	global phxW3OutputUserOptions
	set phxW3OutputUserOptions(stack) {}
	set phxW3OutputUserOptions(parents) $args
}
